/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
/* The following description is used by demos/make-demo-list.pl
 * to compile a list of all tests. */
/* DESCRIPTION
 * silent after a pop [mp3seek test]
 *
 * Function: demo_exactmp3
 */


#include <gtk/gtk.h>
#include <gtk/gtkwidget.h>
#include <gdam/gdambuffer.h>
#include <gdam/gdamsample.h>
#include "../gdamchannel.h"

typedef struct _DemoInfo {
	GdamChannel* channel;
	GtkWindow* top_level;
	GtkLabel* filename;
	GtkLabel* state;
	GtkFileSelection* file_selection;
	GtkAdjustment* samples_lag;
	guint mixer_id;
	guint submix_id;
	guint subscription_id;
	gboolean making_mp3_request;
} DemoInfo;

static void chose_file(GtkButton* button, DemoInfo* info)
{
	char* filename;
	g_return_if_fail(info->file_selection != NULL);
	filename = gtk_file_selection_get_filename(info->file_selection);
	gtk_label_set_text(info->filename, filename);
}

static void filesel_button_clicked(GtkButton* button, DemoInfo* info)
{
	if (info->file_selection) {
		GtkObject* to_kill = GTK_OBJECT(info->file_selection);
		info->file_selection = NULL;
		gtk_object_destroy(to_kill);
	} else {
		GtkWidget* filesel = gtk_file_selection_new("pick your mp3");
		GtkFileSelection* file_select = (GtkFileSelection*)filesel;
		info->file_selection = file_select;
		gtk_signal_connect(GTK_OBJECT(file_select->ok_button),
		                   "clicked",
		                   (GtkSignalFunc) chose_file,
				   info);
		/*gtk_signal_connect_object(
		                   GTK_OBJECT(file_select->cancel_button),
		                   "clicked",
				   (GtkSignalFunc) gtk_widget_destroy,
				   GTK_OBJECT(file_select));*/
		gtk_widget_show(GTK_WIDGET(file_select));
	}
}

static void indicate(DemoInfo* info, const char* state)
{
	gtk_label_set_text(info->state, state);
}

static void stopped_callback(gpointer user_data)
{
	DemoInfo* info = (DemoInfo*)user_data;
	if (info->submix_id == 0 || info->making_mp3_request)
		return;
	info->submix_id = 0;
	indicate(info, "ended");
}
	
static void play_callback(GtkButton* button, DemoInfo* info)
{
	gchar* filename;
	int delta;
	int mp3_id1, mp3_id2, gain_id2;
	GdamTime time_array[4];

	if (info->submix_id != 0 || info->making_mp3_request)
		return;
	if (info->subscription_id) {
		gdam_channel_unsubscribe(info->channel, info->subscription_id);
		info->subscription_id = 0;
	}

	delta = (int)(info->samples_lag->value);

	info->making_mp3_request = TRUE;
	gtk_label_get(info->filename, &filename);
	gdam_debug("create mp3, %s, delta %d", filename, delta);
	mp3_id1 = gdam_channel_create_source_generic(
			info->channel,
			"GdamSeekableMp3",
			1,
			"filename",
			GDAM_ARG_TYPE_STRING,
			filename);
	mp3_id2 = gdam_channel_create_source_generic(
			info->channel,
			"GdamSeekableMp3",
			1,
			"filename",
			GDAM_ARG_TYPE_STRING,
			filename);
	gain_id2 = gdam_channel_create_source_generic(
			info->channel,
			"GdamGain",
			1,
			"gain",
			GDAM_ARG_TYPE_DOUBLE,
			(double)-0.3);
	
	time_array[0] = 0;
	time_array[1] = mp3_id1;
	time_array[2] = 44100;
	time_array[3] = gain_id2;
	info->submix_id = gdam_channel_create_source_generic(
			info->channel,
			"GdamMix",
			2,
			"mixed",
			GDAM_ARG_TYPE_INT64_ARRAY,
			2,
			time_array
	);

	g_assert(info->submix_id != 0);
	gdam_debug("gdam_channel_mix_add being called...");
	if (!gdam_channel_mix_add(
			info->channel,
			info->mixer_id,
			info->submix_id,
			-1))
		g_warning("mix_add failed");
	else
		indicate(info, "playing");
	info->subscription_id = gdam_channel_subscribe_source_ended(
						info->channel, 
						info->submix_id,
						stopped_callback,
						info);
	info->making_mp3_request = FALSE;
}

static void stop_callback(GtkButton* button, DemoInfo* info)
{
	if (info->submix_id == 0 || info->making_mp3_request)
		return;
	if (!gdam_channel_detach_source(info->channel, info->submix_id))
		g_warning("detach source failed");
	else
		indicate(info, "stopped");
	info->submix_id = 0;
}


/* Helper + macro to cut down repetition in demo1_init(). */
static GtkWidget* _generic_button(const char* label, GtkSignalFunc callback,
                                 gpointer user_data)
{
	GtkWidget* button = gtk_button_new_with_label(label);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", callback, user_data);
	gtk_widget_show(GTK_WIDGET(button));
	return button;
}
#define generic_button(label, callback)				\
		_generic_button(label, GTK_SIGNAL_FUNC(callback), info)

void demo_exactmp3(GdamChannel*           channel,
                   guint                  mixer_id)
{
	DemoInfo* info = g_new(DemoInfo, 1);
	GtkWidget* hbox;
	GtkWidget* vbox;
	GtkWidget* widget;
	vbox = gtk_vbox_new(FALSE, 0);
	hbox = gtk_hbox_new(FALSE, 0);

	info->channel = channel;
	info->top_level = (GtkWindow*) gtk_window_new(GTK_WINDOW_TOPLEVEL);
	info->filename = (GtkLabel*) gtk_label_new("/mp3/click/click-1-60.mp3");

	info->state = (GtkLabel*) gtk_label_new("not started");
	info->file_selection = NULL;
	info->making_mp3_request = FALSE;
	info->mixer_id = mixer_id;
	info->submix_id = 0;
	info->subscription_id = 0;
	info->samples_lag = (GtkAdjustment*)gtk_adjustment_new(39, 0, 100000,
	                                                       10,10,10);

	gtk_widget_show(GTK_WIDGET(info->filename));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_WIDGET(info->filename));
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
	                  generic_button("File...", filesel_button_clicked));

	widget = gtk_label_new("Samples delta:");
	gtk_widget_show(widget);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), widget);

	widget = gtk_spin_button_new(info->samples_lag, 10.0, 0);
	gtk_widget_show(widget);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), widget);
	gtk_widget_show(hbox);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_WIDGET(info->state));
	gtk_widget_show(GTK_WIDGET(info->state));

	gtk_box_pack_start_defaults(GTK_BOX(vbox), GTK_WIDGET(hbox));

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
	                        generic_button("Play", play_callback));
	gtk_box_pack_start_defaults(GTK_BOX(hbox),
	                        generic_button("Stop", stop_callback));
	gtk_widget_show(hbox);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), GTK_WIDGET(hbox));
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(info->top_level), GTK_WIDGET(vbox));
	gtk_widget_show(GTK_WIDGET(info->top_level));
}
